
"use strict";

var SliderGallery = new Class({
  
  // TODO: Add support for different navigator types (not only circles, but maby tabs or anything else)

  Implements: [Options, Events],

  options: {
    buttonType: 'text', // possible values: 'text' 'thumb'
    pager: false, // set to selector in case pager required
    nav: '.gallery__nav', // set to false if no nav, inside-gallery selector or to Element
    navItem: 'gallery__nav__item', // If not items inside nav, they will be created empty <a> with folowing class
    sliderOptions: {
      no_mouse_handling: true
    },
    buttonDisabledClass: 'disabled',
    buttonInactiveClass: 'inactive',
    instagram: false,
    disablePrev: true,
    disableNext: false
  },
  slider: undefined,
  nav: undefined,

  initialize: function(gallery, options){
    // First of all lets prevent double initialization
    if (gallery.retrieve("sliderGallery")) return gallery.retrieve("sliderGallery");
    gallery.store("sliderGallery", this);

    var slider = gallery.getElement('.gallery__slider'),
        sliderGalary = this;
    this.setOptions(options);
    this.gallery = gallery;

    this.slider = new SliderFree(slider, (Object.merge?Object.merge:$merge)(this.options.sliderOptions, 
                                                {'onChange': this.onChange.bind(this)}));
    if (!this.slider.slides || this.slider.slides.length==0)
      return;
    // To preveng bugs when element hide when slider loaded, every time someone change its appearence, it also whoud fire 'resize'
    gallery.addEvent('resize', this.updateDimenstions.bind(this));

    // Setting main elements
    this.controls = gallery.getElement('.gallery__arrows');
    this.prevButton = gallery.getElement('.gallery__prev');
    this.nextButton = gallery.getElement('.gallery__next');
    this.pager = this.options.pager ? gallery.getElement(this.options.pager) : undefined;
    this.titleElement = gallery.getElement('.media__title a');
    if (!this.titleElement)
      this.titleElement = gallery.getElement('.media__title');
    this.metaElement = gallery.getElement('.media__meta');
    if (this.options.nav){
      if ((window.typeOf?typeOf:$type)(this.options.nav) === 'element')
        this.nav = this.options.nav;
      else if ((window.typeOf?typeOf:$type)(this.options.nav) === 'string')
        this.nav = gallery.getElement(this.options.nav)
    }
    this.populateNav();
    this.addEvent('change', this.updateControls.bind(this));

    this.attachEvents();
    this.updateControls();

    //
  },

  updateDimenstions: function(){
    this.slider.update_dimensions();
  },

  populateNav: function(){
    if (this.nav){
      var items = this.nav.getElements('.'+this.options.navItem);
      if (items.length===0){
        items = $$();
        for (var i=0;i<this.slider.slides.length; i++){
          var item = new Element('a', {'class': this.options.navItem});
          items.push(item);
          this.nav.adopt(item);
        }
      }
    };
    this.navItems = items;
  },

  updateControls: function(){
    var currentSlide = this.slider.slides[this.slider.current_slide],
        currentSlideId = this.slider.current_slide,
        prevSlide = currentSlide.getPrevious(),
        nextSlide = currentSlide.getNext();

    if (this.titleElement && currentSlide.get('data-title'))
      this.titleElement.set('text', currentSlide.get('data-title'));

    if (this.metaElement && currentSlide.get('data-meta'))
      this.metaElement.set('text', currentSlide.get('data-meta'));

    if (this.nextButton && this.prevButton)
    {
      this.nextButton.removeClass(this.options.buttonDisabledClass);
      this.prevButton.removeClass(this.options.buttonDisabledClass);
      this.nextButton.removeClass(this.options.buttonInactiveClass);
      this.prevButton.removeClass(this.options.buttonInactiveClass);
      if (!nextSlide){
        if (this.options.disableNext)
          this.nextButton.addClass(this.options.buttonDisabledClass);
        else 
          this.nextButton.addClass(this.options.buttonInactiveClass);
      }
      if (!prevSlide){
        if (this.options.disablePrev)
          this.prevButton.addClass(this.options.buttonDisabledClass);
        else 
          this.prevButton.addClass(this.options.buttonInactiveClass);
      }
    }

    if (prevSlide){
      if(this.options.instagram) this.instagram(prevSlide);
      if (this.options.buttonType == 'thumb')
        this.prevButton.getElement('img').set("src", prevSlide.get('data-thumb'));
      else if(this.options.buttonType == 'text')
        this.prevButton.getElement('div').set("text", prevSlide.get('data-title'));
      else if(this.options.buttonType == 'thumb+text'){
        this.prevButton.getElement('span').set("text", prevSlide.get('data-title'));
        this.prevButton.getElement('img').set("src", prevSlide.get('data-thumb'));
      }        
    }
    if (nextSlide){
      if(this.options.instagram) this.instagram(nextSlide);
      if (this.options.buttonType == 'thumb')
        this.nextButton.getElement('img').set("src", nextSlide.get('data-thumb'));
      else if(this.options.buttonType == 'text')
        this.nextButton.getElement('div').set("text", nextSlide.get('data-title'));
      else if(this.options.buttonType == 'thumb+text'){
        this.nextButton.getElement('img').set("src", nextSlide.get('data-thumb'));
        this.nextButton.getElement('span').set("text", nextSlide.get('data-title'));
      }
    }

    this.nav.getElements(".active").removeClass("active");
    this.navItems[currentSlideId].addClass("active");
    if (this.pager)
      this.pager.set('text', ""+(currentSlideId+1)+"/"+this.slider.slides.length)

    if(this.options.instagram) this.instagram(currentSlide);
  },

  attachEvents: function(){
    var sliderGallery = this;
    // Add controls hiding
    if (this.controls)
    {
      var timeoutId;
      this.gallery.addEvent('mouseover', function(e){
        clearTimeout(timeoutId);
        timeoutId=0;
        this.controls.removeClass('hide');
      }.bind(this))
      this.gallery.addEvent('mouseout', function(e){
        timeoutId = setTimeout(function(){this.controls.addClass('hide')}.bind(this), 300);
      }.bind(this))
    }

    if (this.nav && this.navItems){
      sliderGallery.navItems.addEvent('click', function(e){
        if (e) e.stop();
        sliderGallery.showSlide(sliderGallery.navItems.indexOf(this));
      })
    }

    // Set controls events
    this.prevButton.addEvent('click', function(e){
      e.stop();
      if (this.hasClass(sliderGallery.options.buttonDisabledClass))
        return false;
      sliderGallery.showPrev();
    });
    this.nextButton.addEvent('click', function(e){
      e.stop();
      if (this.hasClass(sliderGallery.options.buttonDisabledClass))
        return false;
      sliderGallery.showNext();
    });
  },

  showSlide: function(id){
    this.slider.show_slide(id);
  },

  showNext: function(){
    this.slider.show_next();
  },

  showPrev: function(){
    this.slider.show_prev();
  },
  
  onChange: function(){
    this.fireEvent('change');
  },

  instagram: function(data){
    if(this.options.instagram){
      this.instagramSet(data.get('data-title'),
                        data.get('data-meta'));

    }
  },

  currentSlideId: function(){
    return this.slider.current_slide;
  },

  instagramSet: function(text, data){
    if (text && $$('.instagram__content h2')[0])
      $$('.instagram__content h2')[0].innerText = text;
    if (data && $$('.instagram__content p')[0])
      $$('.instagram__content p')[0].innerText = data;
  }

});


window.addEvent('domready', function(){
  $$(".photo-gallery").each(function(gallery){
    var sliderGallery = new SliderGallery(gallery, {buttonType: 'thumb', pager: ".gallery__page span"});
    // Add cool swiper
    var swiper = window.Code.PhotoSwipe.attach(gallery.getElements('.slide'), {
      'imageScaleMethod': 'fit',
      'loop': false
    });
    gallery.getElement('.media__title,.media__meta').addEvent('click', function(e){
        e.stop();
        swiper.show(sliderGallery.currentSlideId())
      })
    // When swiper opened and window resized sliderGallery will be reRendered and displayd  with 0 height
    swiper.addEventHandler(window.Code.PhotoSwipe.EventTypes.onHide, function(e){sliderGallery.updateDimenstions();} );
  });
  $$(".news-gallery").each(function(gallery){
    new SliderGallery(gallery, {buttonType: 'thumb'});
  });
});


