function SliderFree(element, options){
    this.slider = $(element);
    this.options = (Object.merge?Object.merge:$merge)({}, this.defaults, options || {});
    this.init();
}

SliderFree.prototype = {
  init: function(){
    //this.container = this.slider.getElement('.slide-container');
    this.slider.addClass("slider");
    this.slides = this.slider.getElements(".slide");
    this.container = new Element('div', {'class': 'slide-container'})
                            .adopt(this.slides).inject(this.slider);
    this.bind_events();
    this.current_slide = this.options.init_slide;
    this.update_dimensions();
  },
  bind_events: function(){
    if (('WebKitCSSMatrix' in window && 'm11' in new WebKitCSSMatrix())) {
      this.container.setStyles({
        "-webkit-transform-origin": "0 0",
        "-webkit-transform": "translateZ(0)"
      });
      this.has_transform = true;
    } else {
      this.has_transform = false;
    }
    if ("ontouchstart" in window) {
      this.has_touch = true;
      this.dragstart = "touchstart";
      this.dragmove = "touchmove";
      this.dragend = "touchend"
    } else {
      this.has_touch = false;
      if (this.options.no_mouse_handling){
        this.dragstart = this.dragmove = this.dragend = "";
      } else {
        this.dragstart = "mousedown";
        this.dragmove = "mousemove";
        this.dragend = "mouseup"
      };
    }

    this.ondragstart = this.ondragstart.bind(this);
    this.ondragmove = this.ondragmove.bind(this);
    this.ondragend = this.ondragend.bind(this);
      
    this.container.addEvent(this.dragstart, this.ondragstart);
    this.container.getElements('a').addEvent('click', function(e){if (this.moved){e.stop();}}.bind(this))

    var rotate = 'onorientationchange' in window ? 'orientationchange' : 'resize';
    var rotate_timeout;
    $(window).addEvent(rotate, function () {
      if (rotate_timeout) { clearTimeout(rotate_timeout); }
      rotate_timeout = setTimeout(function () {
        this.update_dimensions();
        rotate_timeout = null;
      }.bind(this), 100);
    }.bind(this));
  },
  ondragstart: function(e){
    if (this.has_touch) {
      var b = e.event.touches;
      if (e.event.touches && e.event.touches.length > 0) {
        var touch = b[0]
      } else {
        return;
      }
    } else {
      var touch = e.event;
      this.moved = false;
      e.preventDefault()
    }
    this.grab();

    document.addEvent(this.dragmove, this.ondragmove);
    document.addEvent(this.dragend, this.ondragend);

    this.init_left = this.last_left = this.get_position();
    this.init_time = this.last_time = (e.timeStamp || new Date().getTime());
    this.last_x = touch.clientX;
    this.init_x = this.last_x//this.last_x + this.init_left - this.current_slide * this.slide_width; // XXX test
    this.last_y = this.init_y = touch.clientY
    this.axis_y_locked = true;

    this.set_position(this.init_left);
  
  },
  ondragmove: function(e){
    if (this.has_touch) {
      if (!e.event.touches || e.event.touches.length > 1) {
        return;
      }
      touch = e.event.touches[0];
      var dY = Math.abs(touch.clientY - this.init_y);
      var dX = Math.abs(touch.clientX - this.init_x);
      if (dY > dX + 3) {
        this.axis_y_locked = false;
        //return true
      }
      if (this.axis_y_locked) {
        e.preventDefault()
      }
    } else {
      touch = e.event;
      this.moved = true;
      e.preventDefault()
    }
    var offset = touch.clientX - this.init_x;
    this.last_move = this.last_x - touch.clientX;
    this.last_x = touch.clientX;

    if (this.current_slide == 0 && offset > 0) {
      offset = Math.sqrt(offset) * 5;
    } else if (this.current_slide == (this.slides.length - 1) && offset < 0) {
      offset = -Math.sqrt(-offset) * 5;
    }

    this.set_position(this.init_left + offset);

    var d = (e.timeStamp || new Date().getTime());
    if (d - this.last_time > 350) {
      this.last_time = d;
      this.last_left = this.init_left + offset;
    }
    return;
    
  },
  ondragend: function(e){
    this.ungrab();

    var left = this.get_position();

    document.removeEvent(this.dragmove, this.ondragmove);
    document.removeEvent(this.dragend, this.ondragend);

    var b = (this.last_left - left);
    var c = Math.max(40, (e.timeStamp || new Date().getTime()) - this.last_time);
    var d = (b!=0 ? Math.abs(b) / c : 0.0001);
    var f = this.slide_width - Math.abs(this.last_left - left);
    var speed = Math.max((f * 1.08) / d, 200);
    speed = Math.min(speed, 600);

    if (this.init_left - 20 > left && this.last_move > 0) {
      this.show_slide(this.current_slide + 1, speed);
    } else if (this.init_left + 20 < left && this.last_move < 0) {
      this.show_slide(this.current_slide - 1, speed);
    } else {
      f = Math.abs(this.last_left - left);
      speed = Math.max((f * 1.08) / d, 200);
      speed = Math.min(speed, 500);
      this.show_slide(this.current_slide, speed)
    }
    return false
  },
  onanimationfinished: function(){
    this.animating = false;
  },

  stop_animation: function(){
    if (this.has_transform) {
      this.container.setStyles({
        '-webkit-transition-duration': '0',
        '-webkit-transition-property': 'none'
      })
    } else {
      var tween = this.container.get('tween');
      if (tween) { tween.pause(); }
    }
    this.onanimationfinished();
  },
  show_slide: function(id, speed, silent){
    id = parseInt(id, 10);
    id = Math.min(this.slides.length-1, id);
    id = Math.max(0, id);

    this.prev_slide = this.current_slide;
    this.current_slide = id;

    speed = speed === undefined? this.options.speed: speed;

    var position = -this.current_slide * this.slide_width;
    if (this.get_position() !== -this.current_slide * this.slide_width) {
      this.set_position(position, speed);
    }
    if(!silent && this.prev_slide != this.current_slide){
      this.options.onChange.call(this);
    }
    this.preload_slides(id);
  },
  preload_slides: function(id){
    var start = Math.max(id-3, 0);
    var stop = Math.min(this.slides.length-1, id+3);
    for (var i=start; i<=stop; i++){
        this.preload_slide(i);
    }
  },
  preload_slide: function(id){
    var data = this.slides[id].getElement('.preload-data');
    var img = this.slides[id].getElement('img');
    if (data && !img){
        function imgload(){
          this.removeEvent('load', imgload);
          data.dispose();
        }
        img = new Element('img').addEvent('load', imgload)
                                .set({'src': data.getProperty('data-src'),
                                      'alt': data.getProperty('data-alt')})
        img.inject(data, 'after');
    }
  },
  show_slide_immediate: function(id){
    return show_slide(id, null);
  },
  show_next: function(){
    this.show_slide(this.current_slide == this.slides.length - 1? 0: this.current_slide + 1);
  },
  show_prev: function(){
    this.show_slide(this.current_slide == 0? this.slides.length - 1: this.current_slide - 1);
  },

  grab: function(){
    this.dragging = true;
    this.container.addClass('dragging');
  },
  ungrab: function(){
    this.dragging = false;
    this.container.removeClass('dragging');
  },

  update_dimensions: function(){
      var w = this.slider.getCoordinates().width;
      var h = this.slider.getCoordinates().height;
      if (w != this.slide_width || h != this.slide_height) {
        this.slide_width = w;
        this.slide_height = h;
        this.slides.setStyles({'height': h, 'width': w});

        this.container.setStyles({
            width: this.slide_width * this.slides.length
        });
        this.show_slide(this.current_slide, null);
      }
      
  },
  
  get_position: function(){
      if (!this.has_transform){
        return parseInt(this.container.getStyle("left"), 10)
      } else {
        // xxx
          var a = window.getComputedStyle(this.container, null).getPropertyValue("-webkit-transform"); 
          var b = a.replace(/^matrix\(/i, '').split(/, |\)$/g);
          return parseInt(b[4], 10)
      }
  },
  set_position: function(left, duration){
    if (!this.has_transform) {
      if (duration){
        this.animating = false;
        this.container.set('tween', {duration: duration,
                                     transition: duration > 0 ? Fx.Transitions.Sine.easeOut 
                                                           : Fx.Transitions.Sine.easeInOut,
                                     onComplete: function () {
                                        this.onanimationfinished();
                                     }.bind(this)
                                    });
        this.container.tween('left', left);
      } else {
          this.container.setStyle("left", left);
      }
    } else {
      if (duration){
        this.animating = false;
        this.container.addEventListener("webkitTransitionEnd", function (e) {
          if (e.target == this.container) {
            this.container.removeEvent("webkitTransitionEnd", arguments.callee);
            this.onanimationfinished();
          }
        }.bind(this), false);
        this.container.setStyles({ 
          '-webkit-transition-duration': duration + "ms",
          '-webkit-transition-property': '-webkit-transform',
          '-webkit-transition-timing-function': (duration > 0 ? "ease-out" : "ease-in-out"),
          '-webkit-transform': 'translate3d(' + left + 'px, 0, 0)'
        });
      } else {
        this.container.setStyles({ 
          '-webkit-transition-duration': '0',
          '-webkit-transition-property': 'none',
          '-webkit-transform': 'translate3d(' + left + 'px, 0, 0)'
        });
      }
    }
  },

  defaults: {
      speed: 400,
      init_slide: 0,
      no_mouse_handling: false,
      onChange: function(){}
  }
}
